unit XApplets;

interface

uses Windows, Messages, XClasses, XWindows, XVisuals;

type
   {$WARNINGS OFF}
   XApplet = class( XWindow )
   private
      fMainWindow : XWindow;
      fCurrentWindow : XWindow;
      fRunning : Boolean;
      fTerminated : Boolean;
      fActive : Boolean;
      fOnMinimize : XOnEvent;
      fOnRestore : XOnEvent;
      fOnActivate : XOnEvent;
      fOnDeactivate : XOnEvent;
      fOnMessage : XOnMessage;
      fOnAddWindow : XOnEvent;
      fWndList : ZList; // of XVisual
      procedure SetCurrentWindow( Value : XWindow );
      function GetIcon : HIcon; override;
   protected
   public
      constructor Create;
      destructor Destroy; override;
      function CreateAppButton : Boolean;
      property MainWindow : XWindow read fMainWindow;
      procedure Run;
      property Runnig : Boolean read fRunning;
      procedure Terminate;
      property Terminated : Boolean read fTerminated;
      procedure Minimize;
      procedure Restore;
      property OnMinimize : XOnEvent read fOnMinimize write fOnMinimize;
      property OnRestore : XOnEvent read fOnRestore write fOnRestore;
      property OnActivate : XOnEvent read fOnActivate write fOnActivate;
      property OnDeactivate : XOnEvent read fOnDeactivate write fOnDeactivate;
      property CurrentWindow : XWindow read fCurrentWindow write SetCurrentWindow;
      property OnMessage : XOnMessage read fOnMessage write fOnMessage;
      property OnAddWindow : XOnEvent read fOnAddWindow write fOnAddWindow;
      function AddChild( Child : XClass ) : Integer; override;
      procedure AddWindowed( Value : XVisual );
      procedure DeleteWindowed( Value : XVisual );
      function WndProc( var Msg : TMsg ) : Integer; override;
   end;
   {$WARNINGS ON}

var
   Applet : XApplet;

   procedure ShowWinNoAnimate( Handle : HWnd; ShowCommand : Integer );

implementation

function GetAnimation: Boolean;
var
  Info: TAnimationInfo;
begin
  Info.cbSize := SizeOf(TAnimationInfo);
  if SystemParametersInfo(SPI_GETANIMATION, SizeOf(Info), @Info, 0) then
    Result := Info.iMinAnimate <> 0 else
    Result := False;
end;

procedure SetAnimation(Value: Boolean);
var
  Info: TAnimationInfo;
begin
  Info.cbSize := SizeOf(TAnimationInfo);
  BOOL(Info.iMinAnimate) := Value;
  SystemParametersInfo(SPI_SETANIMATION, SizeOf(Info), @Info, 0);
end;

procedure ShowWinNoAnimate( Handle : HWnd; ShowCommand : Integer );
var
  Animation: Boolean;
begin
  Animation := GetAnimation;
  if Animation then SetAnimation(False);
  ShowWindow(Handle, ShowCommand);
  if Animation then SetAnimation(True);
end;

constructor XApplet.Create;
begin
   inherited Create( nil );
   fWndList := ZList.Create;
   ClsStyle := CS_OWNDC or CS_VREDRAW or CS_HREDRAW or CS_SAVEBITS;
   Style := Integer( WS_VISIBLE or WS_SYSMENU or WS_POPUP );
   ExStyle := WS_EX_APPWINDOW or WS_EX_DLGMODALFRAME;
   Icon := LoadIcon( hInstance, 'MAINICON' {IDI_APPLICATION} );
   Cursor := LoadCursor( hInstance, IDC_ARROW );
end;

destructor XApplet.Destroy;
var WL : ZList;
begin
   fWndList.Clear;
   WL := fWndList;
   inherited;
   WL.Free;
end;

function XApplet.CreateAppButton : Boolean;
begin
   Result := False;
   if Handle <> 0 then Exit;
   Result := CreateWindow;
end;

{
function XApplet.NewWindow : XWindow;
begin
   if Handle = 0 then
      CreateAppButton;
   Result := XWindow.Create( Self );
   if fMainWindow = nil then
      fMainWindow := Result;
end;
}

procedure XApplet.Run;
var Msg : TMsg;
begin
   if Handle = 0 then
      if not CreateAppButton then Exit;
   fRunning := True;
   if fMainWindow = nil then Exit;
   if fMainWindow.Handle = 0 then
      if not fMainWindow.CreateWindow then Exit;
   while GetMessage( Msg, 0, 0, 0 ) do
   begin
      TranslateMessage( Msg );
      DispatchMessage( Msg );
      if Msg.hwnd <> Handle then
      if Msg.hwnd <> fMainWindow.Handle then
      if Terminated then break;
   end;
   fRunning := False;
   Applet.Free;
   Applet := nil;
end;

procedure XApplet.Terminate;
begin
   if not fRunning then Exit;
   if Terminated then Exit;
   fTerminated := True;
end;

function XApplet.WndProc( var Msg : TMsg ) : Integer;
var Handled : Boolean;
    I : Integer;
    V : XVisual;
    H : HWnd;
  {
  procedure DrawAppIcon;
  var
    DC: HDC;
    PS: TPaintStruct;
  begin
    begin
      DC := BeginPaint( Msg.hwnd, PS);
      DrawIcon(DC, 0, 0, Icon );
      EndPaint( Msg.hwnd, PS);
    end;
  end;
  }
begin
   if Assigned( fOnMessage ) then
   begin
      Handled := False;
      fOnMessage( Msg, Result, Handled );
      if Handled then Exit;
   end;
   if Msg.hwnd = Handle then
   case Msg.message of
   WM_CLOSE:
              begin
                 fTerminated := True;
                 Result := 0;
                 Exit;
              end;
   WM_ACTIVATEAPP: begin
                      Result := inherited WndProc( Msg );
                      fActive := Msg.wParam <> 0;
                      if fActive then
                      begin
                         if MainWindow <> nil then
                            Restore;
                         PostMessage( Handle, CM_ACTIVATE, 0, 0 );
                      end
                         else
                      begin
                         PostMessage( Handle, CM_DEACTIVATE, 0, 0 );
                      end;
                      Exit;
                   end;
   CM_ACTIVATE:
      if Assigned(FOnActivate) then FOnActivate(Self);
   CM_DEACTIVATE:
      begin
         if Assigned(FOnDeactivate) then FOnDeactivate(Self);
      end;
   WM_SysCommand: begin
                     case Msg.wParam and $FFF0 of
                        SC_MINIMIZE: begin
                                        Minimize;
                                        Result := 0;
                                        Exit;
                                     end;
                        SC_RESTORE:  begin
                                        Restore;
                                        Result := 0;
                                        Exit;
                                     end;
                     end;
                  end;
   WM_SETFOCUS:
      PostMessage( Handle, CM_ENTER, 0, 0 );
   CM_ENTER:
      begin
         if not IsIconic( Handle ) and (GetFocus = Handle) then
         begin
            if CurrentWindow <> nil then
               SetFocus( CurrentWindow.Handle )
            else
            if MainWindow <> nil then
               SetFocus( MainWindow.Handle );
         end;
      end;
   {
   WM_PAINT:
      if IsIconic( Handle ) then
      begin
         DrawAppIcon;
         Result := 0;
         Exit;
      end;

   WM_QUERYDRAGICON:
      begin
         Result := Icon;
      end;
   }
   end
      else
   begin
      for I := 0 to fWndList.Count - 1 do
      begin
         V := fWndList.Items[ I ];
         H := XVisuals.GetWindowHandle( V );
         if (H = HWnd(-1)) or (H = Msg.hwnd) then
         begin
            Result := (V as XVisual).WndProc( Msg );
            Exit;
         end;
      end;
   end;
   Result := inherited WndProc( Msg );
end;

procedure XApplet.Minimize;
begin
   if Handle = 0 then Exit;
   if not IsIconic( Handle ) then
   begin
      SetActiveWindow( Handle );
      ShowWinNoAnimate( Handle, SW_MINIMIZE );
      if Assigned( fOnMinimize ) then
         fOnMinimize( Self );
   end;
end;

procedure XApplet.Restore;
begin
   if IsIconic( Handle ) then
   begin
      SetActiveWindow( Handle );
      ShowWinNoAnimate( Handle, SW_RESTORE );
      if (MainWindow <> nil) and (MainWindow.WindowState = xwsMinimized)
      and not MainWindow.Visible then
      begin
         MainWindow.WindowState := xwsNormal;
         MainWindow.Show;
      end;
      if Assigned( fOnRestore ) then
         fOnRestore( Self );
   end;
end;

procedure XApplet.SetCurrentWindow( Value : XWindow );
begin
   if Value.Parent <> Self then Exit;
   fCurrentWindow := Value;
end;

function XApplet.AddChild( Child : XClass ) : Integer;
begin
   Result := inherited AddChild( Child );
   if Result >= 0 then
   if Child is XWindow then
   begin
      if fMainWindow = nil then
         fMainWindow := Child as XWindow;
      if Assigned( fOnAddWindow ) then
         fOnAddWindow( Self );
   end;
end;

function XApplet.GetIcon : HIcon;
begin
   Result := inherited GetIcon;
   if Result = 0 then
      Result := LoadIcon( hInstance, IDI_APPLICATION );
end;

procedure XApplet.AddWindowed(Value: XVisual);
begin
   if Value = Self then Exit;
   if not Value.Windowed then Exit;
   DeleteWindowed( Value );
   fWndList.Add( Value );
end;

procedure XApplet.DeleteWindowed(Value: XVisual);
var Idx : Integer;
begin
   Idx := fWndList.IndexOf( Value );
   if Idx < 0 then Exit;
   fWndList.Delete( Idx );
end;

end.
